/* vim: set sw=8 ts=8 si : */
/*
* Author: Guido Socher
* Copyright: GPL V2
*/
#include <avr/io.h>


int convertanalog(unsigned char channel,unsigned char iref) 
{
	unsigned char adlow,adhigh;
        /* enable analog to digital conversion in single run mode
        *  without noise canceler function. See datasheet of atmega88 page 250
        * We set ADPS2=1 to have a clock division factor of 16.
        * This is needed to stay in the recommended range of 50-200kHz 
	* Clock freq=1MHz
        * ADEN: Analog Digital Converter Enable
        * ADIE: ADC Interrupt Enable  (0=no interrupt)
        * ADIF: ADC Interrupt Flag
        * ADCSR: ADC Control and Status Register
        * ADPS2..ADPS0: ADC Prescaler Select Bits
	* REFS: Reference Selection Bits (page 203)
        */

	// int-ref with external capacitor at AREF pin: 
	// 2.56V int ref=REFS1=1,REFS0=1
	// write only the lower 3 bit for channel selection
	//
        if (iref){
		// Use the intenal 2.56V ref:
		ADMUX=(1<<REFS1)|(1<<REFS0)|(channel & 0x0f);
        }else{
		// AVcc ref
		ADMUX=(1<<REFS0)|(channel & 0x0f);
	}
	// 
        ADCSR=(1<<ADEN)|(1<<ADPS2);
	// switch off digital input line:
	//DIDR0=(1<<channel)& 0x1f;

	//  start conversion 
	ADCSR|= (1<<ADSC);
	while(bit_is_set(ADCSR,ADSC)); // wait for result 
	adlow=ADCL; // read low first !! 
	adhigh=ADCH; 
	return((adhigh<<8)|(adlow & 0xFF));
}


